unit TestSuite;

interface
{
c. Mike Johnson - Big Attic House (father@bigattichouse.com) 2002.

The idea is to provide a more Delphi-App Coder friendly testing suite
similar to dUnit for all the folks not quite into OO programming.
This way you can build a datamodule, or place tests on each form.

In the next version, all the tests will be conglomerated into a form
that will be visible when you are building your project, to automate the
testing.. super simple, and Delphi friendly.

You may use this software freely in your projects... it is freeware. Future
commercial versions may be created as pay-only.  If you update this source, fix bugs,
etc - please send me a copy so I can update the project for all to use.

Pay Forward Please!

Version 1.6:
  Changes:
  1. qinghai2000@sina.com - Thread Safety in Global tests, XML and HTML Consoles.
Version 1.5:
  Changes:
  1. Kyle Cordes : Sender:TQtest to be more Semantically correct Test:TQtest in the TRunQTest procedure
  2. Added some "Expect" routines.
  3. Added a list of all statements executed (info lines from Expect and Check)
}
uses
  SysUtils, Classes, syncobjs
  {$IFDEF LINUX}
    ,libc
  {$ELSE}
    ,windows,graphics
  {$ENDIF};

type

  TQTest=class;{forward}

  TQTestResult = (qtUntested,qtPass,qtFail,qtException,qtLog,qtTimer,qtScreenshot);
  TRunQTest = procedure (Test:TQTest;var Result:TQTestResult) of object;

  TQTime = class(TObject)
    private
      ftime,fstarttime:longword;
      fname:String;
    public
      property Name:String read fname write fname;
      property StartTime:longword read fstarttime write fstarttime;
      property ElapsedTime:longword read ftime write ftime;
    end;

  TQTest = class(TCollectionItem)
   private
    fOnTest:TRunQTest;  fname,fdetails:String; fresult:TQTestResult;
    fStatements:TStringlist;
       fsetup,fteardown:TNotifyEvent;
       fshowtime:boolean;
       ftimers:tstringlist;
   protected
     procedure SetDisplayName(const Value: string);override;
     function  GetDisplayName:String;override;
     function  GetTime:longword;
     function FindTimer(name:String):TQTime;
   public
     function  GetNamePath:String;override;
     function  Test:TQTestResult;

     procedure Check (info:String;IsTrue:boolean);
     procedure Log(info,detail:String);
     procedure ExpectInteger (info:String;Expected,Found:integer);
     procedure ExpectDate (info:String;Expected,Found:tdatetime);
     procedure ExpectString (info:String;Expected,Found:string);
     procedure ExpectMoney (info:String;Expected,Found:currency);
     procedure ExpectFloat (info:String;Expected,Found:extended);
     procedure BenchmarkOverInteger (info:String;Mark,Found:integer);
     procedure BenchmarkOverFloat   (info:String;Mark,Found:extended);
     procedure BenchmarkUnderInteger (info:String;Mark,Found:integer);
     procedure BenchmarkUnderFloat   (info:String;Mark,Found:extended);

     {$IFDEF LINUX}
   {IGNORE UNTIL I FIND SOME SCREENSHOT CODE FOR LINUX}
     {$ELSE}
       procedure ScreenShot (directory,caption,details:String);overload;
       procedure ScreenShot (WindowHandle:THandle;width,height:word;directory,caption,details:String);overload;
     {$ENDIF}

     procedure ResetTimer(name:string);
     procedure StartTimer(name:string);
     function  StopTimer(name:string):longword;
     procedure LogTimer(name:string);

     property  Details:String read fDetails;
     function  StatementCount:integer;
     function  Statements(index:integer):String;
     function  StatementResults(index:integer):TQTestResult;
     constructor create(Collection:TCollection);override;
     destructor  destroy;override;
   published
     property  TestName:String read getdisplayname write setdisplayname;
     property  ShowTimestamp:boolean read fshowtime write fshowtime;
     property  OnTest:TRunQTest read fOnTest write fOnTest;
     property  Result:TQTestResult read fresult write fresult;
     property OnSetup:TNotifyEvent read fsetup write fsetup;
     property OnTearDown:TNotifyEvent read fteardown write fteardown;
   end;

  TQTestCollection = class(TCollection)
  private
    fowner:TPersistent;
  protected
    function GetItem(index:integer):TQTest;
    procedure SetItem(index:integer;Item:TQTest);
  public
    function GetOwner:TPersistent;override;
    constructor Create(Aowner:TComponent);
    function Add: TQTest;
    property Items[Index: Integer]: TQtest
      read GetItem write SetItem; default;
  end;

  TTestSuite = class(TComponent)
  private
    ftests:TQTestCollection;
    fsetup,fteardown:TNotifyEvent;
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor create(aowner:TComponent);override;
    destructor destroy;override;
  published
    { Published declarations }
    property  Tests:TQTestCollection read ftests write ftests;
    procedure ExecuteTests;
    property OnSetup:TNotifyEvent read fsetup write fsetup;
    property OnTearDown:TNotifyEvent read fteardown write fteardown;
  end;

{New in v1.6 ... play with the data as you wish!}
TBasicConsole = class(TComponent)
  private
    ftestlist:TList;
  public
    constructor Create(Aowner:TComponent);override;
    destructor Destroy;override;
    procedure AddSuite(Test:TTestSuite);
    procedure AddAllSuites;
    procedure AddSuiteByName(Module,Name:String);
    function  SuiteCount:integer;
    function  Suites(index:integer):TTestSuite;
    procedure DeleteSuite(index:integer);
    procedure Execute; virtual;
  end;

procedure RegisterTestSuite (Test:TTestSuite);
procedure UnRegisterTestSuite (Test:TTestSuite);
function  TestSuiteCount:integer;
function  TestSuites(index:integer):TTestSuite;
function  TestSuiteByName(Module,Name:String):TTestSuite;



implementation

{$R *.DCR}

var GlobalTests:Tstringlist;GlobalTestsLock:TCriticalSection;

procedure LockGlobalTests;
begin
  GlobalTestsLock.Acquire;
end;

procedure UnLockGlobalTests;
begin
  GlobalTestsLock.Release;
end;

procedure RegisterTestSuite (Test:TTestSuite);
var TestName:String;
begin
 if assigned(test) then begin
  LockGlobalTests;
    try
       TestName:=Test.Name+'='+Test.Owner.name;
       if GlobalTests.IndexOfObject(Test)=-1 then GlobalTests.addobject (TestName,Test);
     finally
       UnLockGlobalTests;
     end;
 end;
end;

procedure UnRegisterTestSuite (Test:TTestSuite);
var idx:integer;
begin
 if assigned(test) then begin
   LockGlobalTests;
    try
       idx:=Globaltests.IndexOfObject(Test);
       if idx>-1 then Globaltests.delete(idx);
    finally
       UnLockGlobalTests;
    end;
 end;
end;

function TestSuiteCount:integer;
begin
   LockGlobalTests;
   result:=GlobalTests.count;
   UnLockGlobalTests;
end;

function  TestSuites(index:integer):TTestSuite;
begin
 LockGlobalTests;
  result:=TTestSuite(GlobalTests.objects[index]);
 UnLockGlobalTests;  
end;

function  TestSuiteByName(Module,Name:String):TTestSuite;
var idx:integer;
begin
 LockGlobaltests;
 idx:= GlobalTests.indexof(Name+'='+Module);
 result:=nil;
 if idx>-1 then Result:=TTestSuite(GlobalTests.objects[idx]);
 UnLockGlobalTests;
end;

constructor TQTest.create(Collection:TCollection);
begin
 inherited create(collection);
 fshowtime:=false;
 fstatements:=tstringlist.create;
 ftimers:=tstringlist.create;
end;

destructor  TQTest.destroy;
begin
  fstatements.free;
  while ftimers.count>0 do begin
     TQTime(ftimers.objects[0]).free;
     ftimers.delete(0);
  end;
  inherited destroy;
end;

function  TQTest.StatementCount:integer;
begin
 result:=fstatements.count;
end;

function  TQTest.Statements(index:integer):String;
begin
  result:=fstatements[index];
end;

function  TQTest.StatementResults(index:integer):TQTestResult;
begin
  result:=TQTestResult(fstatements.Objects[index]);
end;

procedure  TQTest.Check (info:String;IsTrue:boolean);
var StatementResult:TQTestResult; outhold:String;
begin
 if ((fresult=qtUntested) or (fresult=qtPass)) then
   if IsTrue then fresult:=qtPass
      else fresult:=qtFail;

 if IsTrue then StatementResult:=qtPass else StatementResult:=qtFail;

 outhold:=info;
 if fshowtime then outhold:='('+Datetimetostr(now())+') '+info;

 fstatements.addobject(outhold,pointer(ord(StatementResult)));
end;

function TQtest.GetTime: longword;
begin
{$IFDEF LINUX}
  result := clock div (CLOCKS_PER_SEC div 1000);
{$ELSE}
  result := Windows.GetTickCount;
{$ENDIF}
end;


{$IFDEF LINUX}

{$ELSE}
procedure TQtest.ScreenShot (directory,caption,details:String);
begin
  Screenshot( GetDesktopWindow,GetSystemMetrics(SM_CXSCREEN), GetSystemMetrics(SM_CYSCREEN),directory,caption,details);
end;

procedure TQtest.ScreenShot (WindowHandle:THandle;width,height:word;directory,caption,details:String);
  var outhold:String;
      DCDesk: HDC; // hDC of Desktop
      bmp: TBitmap;w,h:longint;  top,left:integer;
begin
  {Create a bitmap}
  bmp := TBitmap.Create;
   
  {Set a bitmap sizes}
  
  w:=width;
  h:=height;
  bmp.Height :=h;{avoiding screen.height to trim forms from uses clause}
  bmp.Width := w;

  {Get a desktop DC handle - handle of a display device context}
  DCDesk := windows.GetDC(WindowHandle) ;

  top:=0;left:=0;
  if WindowHandle<>GetDesktopWindow then begin
      top:=-4; left:=-24;
  end;

  {Copy to any canvas, here canvas of an image}
  BitBlt(bmp.Canvas.Handle, 0, 0, w, h,
         DCDesk, top, left, SRCCOPY);

  {Save the bitmap}
  if extractfilepath(directory)<>directory then directory:=directory+'\';
  bmp.SaveToFile(directory+caption+'.bmp');

  outhold:= 'Screenshot:'+caption+' '+details;
  if fshowtime then outhold:='('+Datetimetostr(now())+') '+outhold;
  fstatements.addobject(outhold+':'+' <A HREF="'+directory+caption+'.bmp'+'" TARGET="_POP"><IMG SRC="'+directory+caption+'.bmp'+'" WIDTH=100></A>',pointer(ord(qtScreenshot)));

  {Release desktop DC handle}
  ReleaseDC(GetDesktopWindow, DCDesk);

  {Release a bitmap}
  bmp.Free;
end;


{$ENDIF}


function TQtest.FindTimer(name:String):TQTime;
var idx:integer;
begin
 result:=nil;
 idx:=ftimers.indexof(UPPERCASE(Trim(name)));
 if idx>-1 then
    result:=TQTime(ftimers.objects[idx])
    else begin
      result:=TQTime.create;
      result.name:=name;
      result.ElapsedTime:=0;
      result.StartTime:=0;
      ftimers.addobject(UPPERCASE(Trim(name)),result);
    end;
end;

procedure TQtest.ResetTimer(name:string);
var T:TQtime;
begin
 T:=findTimer(name);
 T.ElapsedTime:=0;
 T.StartTime :=0;
end;

procedure TQtest.StartTimer(name:string);
var T:TQtime;
begin
 T:=findTimer(name);
 T.StartTime:=GetTime;
end;

function TQtest.StopTimer(name:string):longword;
var T:TQtime;  tick:longword;
begin
 tick:=GetTime;
 T:=findTimer(name);
 T.ElapsedTime:=T.ElapsedTime+ (tick-T.StartTime);
 result:= T.ElapsedTime;
end;

procedure TQtest.LogTimer(name:string);
var outhold:String;
    T:TQtime;  
begin
  T:=findTimer(name);
  StopTimer(name);
  outhold:=name;
  if fshowtime then outhold:='('+Datetimetostr(now())+') '+name;
  fstatements.addobject(outhold+':'+inttostr(T.ElapsedTime)+' ms',pointer(ord(qtTimer)));
end;

procedure TQTest.Log(info,detail:String);
var outhold:String;
begin
  outhold:=info;
  if fshowtime then outhold:='('+Datetimetostr(now())+') '+info;
  fstatements.addobject(outhold+':'+Detail,pointer(ord(qtLog)));
end;

procedure TQTest.ExpectInteger (info:String;Expected,Found:integer);
begin
 Check(info + ' expected:' + inttostr(Expected) + ' found:'+ inttostr(Found),expected=Found);
end;

procedure TQTest.ExpectDate (info:String;Expected,Found:tdatetime);
begin
 Check(info + ' expected:' + datetimetostr(Expected) + ' found:'+ datetimetostr(Found),expected=Found);
end;

procedure TQTest.ExpectString (info:String;Expected,Found:string);
begin
 Check(info + ' expected:' + Expected + ' found:'+ Found,expected=Found);
end;

procedure TQTest.ExpectMoney (info:String;Expected,Found:currency);
begin
 Check(info + ' expected:' + CurrToStr(Expected) + ' found:'+ CurrTostr(Found),expected=Found);
end;

procedure TQTest.ExpectFloat (info:String;Expected,Found:extended);
begin
 Check(info + ' expected:' + FloatToStr(Expected) + ' found:'+ FloatToStr(Found),expected=Found);
end;

procedure TQtest.BenchmarkOverInteger (info:String;Mark,Found:integer);
begin
 Check(info + ' benchMark Over:' + inttostr(Mark) + ' found:'+ inttostr(Found),Found>=Mark);
end;

procedure TQtest.BenchmarkOverFloat   (info:String;Mark,Found:extended);
begin
 Check(info + ' benchMark Over:' + FloatToStr(Mark) + ' found:'+ FloatToStr(Found),Found>=Mark);
end;

procedure TQtest.BenchmarkUnderInteger (info:String;Mark,Found:integer);
begin
 Check(info + ' benchmark under:' + inttostr(Mark) + ' found:'+ inttostr(Found),Found<=Mark);
end;

procedure TQtest.BenchmarkUnderFloat   (info:String;Mark,Found:extended);
begin
 Check(info + ' benchmark under:' + FloatToStr(Mark) + ' found:'+ FloatToStr(Found),Found<=Mark);
end;

function  TQTest.Test :TQTestResult;
var outhold:String;
begin
  fstatements.clear;
  fdetails:='';
  fresult:=qtUnTested;
try
  try
  if assigned(fsetup) then fsetup(self);
  if assigned(fontest) then fOnTest(self,fresult);
  except on E:Exception do
   begin
     fresult:=qtException;
     fdetails:='';
      outhold:=E.Message;
      if fshowtime then outhold:='('+Datetimetostr(now())+') '+E.Message;
     fstatements.addobject(outhold,pointer(ord(qtException)));
   end;
   end;
finally
   if assigned(fteardown) then fteardown(self);
end;
  result:=fresult;
end;

procedure TQTest.SetDisplayName(const Value: string);
begin
 fname:=Value;
end;

function  TQTest.GetNamePath:String;
begin
 result:=TComponent(TQTestcollection(self.collection).getowner).name+'.'+fname;
end;

function TQTest.GetDisplayName:String;
begin
 if fname='' then fname:=(inherited GetDisplayname);
 result:=fname;
end;

constructor TTestSuite.create(aowner:TComponent);
begin
 inherited create(Aowner);
 ftests:=TQTestCollection.Create(self);
 if assigned(aowner) then RegisterTestSuite(self);
end;

destructor TTestSuite.destroy;
begin
  try
    UnRegisterTestSuite(self);
    ftests.clear;
    ftests.free;
  finally
    inherited destroy;
  end;
end;

procedure  TTestSuite.ExecuteTests;
var max,i:integer;
begin
 max:= ftests.count-1;
 if assigned(fSetup) then fSetup(self);
for i:= 0 to max do
 begin
   //Try
   ftests[i].Test;
  // except on E:Exception do
  // begin
  //  ftests[i].Result := qtException;
  // end;end;

 end;
 if assigned(fTeardown) then fteardown(self);
end;

function   TQTestCollection.GetItem(index:integer):TQTest;
begin
 Result := TQTest(inherited GetItem(Index));
end;

procedure   TQTestCollection.SetItem(index:integer;Item:TQTest);
begin
  inherited SetItem(Index, Item);
end;

function TQTestCollection.GetOwner:TPersistent;
begin
  result:=fowner;
end;

constructor TQTestCollection.Create (Aowner:TComponent);
begin
  inherited create(TQTest);
  fowner:=Aowner;
end;

function    TQTestCollection.Add: TQTest;
begin
  Result := TQTest(inherited Add);
end;


constructor TBasicConsole.Create(Aowner:TComponent);
begin
   inherited create(Aowner);
    ftestlist:=TList.create;
end;

destructor TBasicConsole.Destroy;
begin
  try
    ftestlist.free;
  finally
    inherited Destroy;
  end;
end;

procedure TBasicConsole.AddSuite(Test:TTestSuite);
begin
 if assigned(Test) then ftestlist.add(Test);
end;

procedure TBasicConsole.AddAllSuites;
var max,i:integer;
begin
 max:=TestSuiteCount-1;
 for i:= 0 to max do begin
   AddSuite(TestSuites(i));
 end;
end;

procedure TBasicConsole.AddSuiteByName(Module,Name:String);
begin
  AddSuite(TestSuiteByName(Module,Name));
end;

function  TBasicConsole.SuiteCount:integer;
begin
  result:=ftestlist.count;
end;

function  TBasicConsole.Suites(index:integer):TTestsuite;
begin
 result:=TTestSuite(ftestlist[index]);
end;

procedure TBasicConsole.DeleteSuite(index:integer);
begin
 ftestlist.delete(index);
end;

procedure TBasicConsole.Execute;
var max,i:integer;Q:TTestSuite;
begin
  max:=ftestlist.count-1;
  for i:=0 to max do begin
    Q:=Ttestsuite(ftestlist[i]);
    try
     Q.ExecuteTests ;
    except on E:Exception do
    end;
  end;
end;

initialization
 GlobalTestsLock:=TCriticalSection.create;
 GlobalTests:=Tstringlist.create;
finalization
 {don't free items, they are owned by their forms..}
 GlobalTests.free;
 GlobalTestsLock.free;
end.
