unit TestSuiteConsole;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, ImgList, TestSuite;

type
{
version 1.7 (Jan 2,2003) :
      Much prettier images in console
      Created a TBasicConsole base class for creating your own consoles..
      Created a TWebConsole that can output as HTML or XML,
      New thread safety makes Qtest viable for web applications...
       (uses TCriticalSection, let me know if you have problems)
      Added "Benchmark" Checks, where you want some value to be > or < a Mark
       (like page generated under a certain # of milliseconds).
      Created sample CGI app to show how to use Qtest in ISAPI/WEBBroker/CGI.
      Fixed a few glitches.

version 1.6 (Dec 11,2002):
       Some refactoring by qinghai2000@sina.com

version 1.5 (Nov 7,2002):
       Allows for single test execution,
       Shows all statements (info lines from expect and check functions)
       Anchors for better resizing
       A few other details.

With Special Thanks to:
        Peter Burn (burn@pentag.ch) most of the ideas/mods for v1.5
        Antony dn Dulk (Antony.Dulk@vababv.com) pointing out a glaring error.
        <qinghai2000@sina.com> refactoring.
        Phillip Flores <pflores@pcfworks.com> for ideas.
}

  TTestConsole = class(TForm)
    btnRunAllTests: TButton;
    ImageList1: TImageList;
    btnCancel: TButton;
    Passcount: TLabel;
    TestTree: TTreeView;
    btnRunSingleTest: TButton;
    cboTestList: TComboBox;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnCancelClick(Sender: TObject);
    procedure btnRunAllTestsClick(Sender: TObject);
    procedure btnRunSingleTestClick(Sender: TObject);
    procedure TestTreeGetSelectedIndex(Sender: TObject; Node: TTreeNode);
    procedure TestTreeDblClick(Sender: TObject);
  private
    { Private declarations }
    function FindNodeByTest(aTest: TQTest): TTreeNode;
    procedure UpdatePassCountResultByTest(aTest: TQTest;
      var aPassedCount: integer; aAllTestsCount: integer);
    procedure FillNodeChildsByTest(var aNode: TTreeNode; aTest: TQTest);
    procedure UpdateNodeByTest(var aNode: TTreeNode; aTest: TQTest);

    procedure UpdateByTest(aTest:TQTest;Var aPassedCount:integer;aAllTestsCount:integer);
  public
    { Public declarations }
    procedure Execute;
  end;

procedure RunQTests;

implementation

{$R *.DFM}

var
  TestConsole: TTestConsole;

procedure RunQTests;
begin
  if not (assigned(TestConsole)) then
    Application.CreateForm(TTestConsole, TestConsole);
  TestConsole.execute;
  TestConsole.Show;
end;

procedure TTestConsole.Execute;
var
  forms: TStringList;

  procedure AddTestSuitesToForms;
  var
    i: integer;
    thisTestSuite: TTestSuite;
  begin
    Passcount.caption := '';
    TestTree.items.Clear;

    while forms.count > 0 do
    begin
      forms.delete(0);
    end;
    TestTree.items.clear;

    forms.Sorted := true;

    for i := 0 to TestSuiteCount - 1 do
    begin
      thisTestSuite := TestSuites(i);
      forms.Add(thisTestSuite.owner.name);
    end;
  end;

  procedure AddFormsToTree;
  var
    i: integer;
    formNode: TTreeNode;

  begin
    {Add forms to tree}
    for i := 0 to forms.count - 1 do
    begin
      formNode := TestTree.items.add(nil, forms[i]);
      formNode.ImageIndex := 6;
      forms.objects[i] := formNode;
    end;
  end;

  procedure AddTestToTreeNode;
  var
    i, j, idx: integer;
    thisTestSuite: TTestSuite;
    formNode, suiteNode, testNode: TTreeNode;
  begin
    for i := 0 to TestSuiteCount - 1 do
    begin
      thisTestSuite := TestSuites(i);
      idx := forms.IndexOf(thisTestSuite.owner.name);
      formNode := TTreeNode(forms.objects[idx]);
      suiteNode := TestTree.items.addchildobject(formNode, thisTestSuite.Name, thisTestSuite);
      suiteNode.ImageIndex := 5;
      
      for j := 0 to thisTestSuite.Tests.count - 1 do
      begin
        testNode := TestTree.items.addchildobject(suiteNode, thisTestSuite.Tests[j].TestName, thisTestSuite.Tests[j]);
        cboTestList.items.addobject(thisTestSuite.Name + '.' + thisTestSuite.Tests[j].TestName, thisTestSuite.Tests[j]);
        testNode.ImageIndex := 0;
      end
    end;
    TestTree.FullExpand;
  end;

  procedure AddTestToCboTestList;
  var
    i, j: integer;
    thisTestSuite: TTestSuite;
  begin
    cbotestlist.clear;
    for i := 0 to TestSuiteCount - 1 do
    begin
      thisTestSuite := TestSuites(i);
      for j := 0 to thisTestSuite.Tests.count - 1 do
      begin
        cboTestList.items.addobject(thisTestSuite.Name + '.' + thisTestSuite.Tests[j].TestName, thisTestSuite.Tests[j]);
      end;
    end;

    if cboTestlist.items.count > 0 then
    begin
      cboTestlist.ItemIndex := 0;
      cboTestlist.text := cbotestlist.items[0];
    end;
  end;

begin
  {Draw that there Tree View!}
  forms := TStringList.create;
  try
    AddTestSuitesToForms;

    AddFormsToTree;

    AddTestToTreeNode;

    AddTestToCboTestList;
  finally
    forms.free;
  end;
end;

procedure TTestConsole.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action := caHide;
end;

procedure TTestConsole.btnCancelClick(Sender: TObject);
begin
  Close;
end;

procedure TTestConsole.btnRunAllTestsClick(Sender: TObject);
var
  i,j:integer;
  thisTestSuite:TTestSuite;
  thisTest:TQTest;
  AllTestsCount, PassedCount: integer;

  function GetAllTestsCount:integer;
  var
    i:integer;
    thisTestSuite:TTestSuite;
  begin
    Result:=0;
    for i := 0 to TestSuiteCount - 1 do
    begin
      thisTestSuite := TestSuites(i);
      Result := Result + thisTestSuite.Tests.count;
    end;
  end;
begin
 {Run Tests}
  AllTestsCount := GetAllTestsCount;
  Passcount.caption := '0% Passed (0/'+inttostr(AllTestsCount)+').';
  PassedCount := 0;

  for i := 0 to TestSuiteCount - 1 do
  begin
    thisTestSuite := TestSuites(i);
    TestSuites(i).ExecuteTests;

    for j := thisTestSuite.Tests.count - 1 downto 0 do
    begin
      thisTest:=thisTestSuite.Tests[j];
      UpdateByTest(ThisTest,PassedCount,AllTestsCount);
    end
  end;
  TestTree.FullExpand;
  TestTree.Refresh;
  application.processmessages;
end;

procedure TTestConsole.UpdatePassCountResultByTest(aTest: TQTest; var aPassedCount: integer; aAllTestsCount: integer);
begin
  if aTest.result = qtPass then
  begin
    inc(aPassedCount);
    Passcount.caption := inttostr(round(100 * (aPassedCount / aAllTestsCount))) + '% Passed ('+inttostr(aPassedCount)+'/'+inttostr(aAllTestsCount)+').'
  end;
end;

procedure TTestConsole.FillNodeChildsByTest(var aNode: TTreeNode; aTest: TQTest);
var
  i: integer;
begin
  aNode.DeleteChildren;
  application.processmessages;

  for i := 0 to aTest.StatementCount - 1 do begin
    If aTest.Statements(i) <> '' then begin
       TestTree.items.AddChild(aNode, aTest.Statements(i));
      aNode.Item[i].ImageIndex := ord(aTest.StatementResults(i));
    end;
  end;
end;

procedure TTestConsole.UpdateNodeByTest(var aNode: TTreeNode; aTest: TQTest);
begin
  aNode.ImageIndex := ord(aTest.result);
  if aTest.result <> qtUntested then
    aNode.Text := aTest.TestName;
  if aTest.details <> '' then aNode.Text := aTest.TestName + ' (' + aTest.Details + ')';
end;

procedure TTestConsole.btnRunSingleTestClick(Sender: TObject);
var
  AllTestsCount, PassedCount: integer;
  ThisTest: TQTest;
  ThisNode: TTreeNode;

begin
 {Run Tests}
  Passcount.Caption := '0% PassedCount.';
  AllTestsCount := 1;
  PassedCount := 0;

    {Find our testNode on the tree and update its image}
  if cboTestlist.itemindex > -1 then
    if assigned(cboTestList.items.objects[cboTestlist.itemindex]) then
    begin
      ThisTest := TQTest(cboTestList.items.objects[cboTestlist.itemindex]);

      ThisTest.test;

      UpdateByTest(ThisTest,PassedCount,AllTestsCount);
    end;

  TestTree.FullExpand;
end;

function TTestConsole.FindNodeByTest(aTest: TQTest): TTreeNode;
var
  i: integer;
begin
  Result := nil;

  for i := 0 to TestTree.Items.Count - 1 do
  begin
    if TestTree.Items[i].Data = aTest then
    begin
      Result := TestTree.Items[i];
      Break;
    end;
  end;
end;

procedure TTestConsole.UpdateByTest(aTest: TQTest;
  var aPassedCount: integer; aAllTestsCount: integer);
var
  ThisNode:TTreeNode;
begin
  ThisNode := findNodeByTest(aTest);

  UpdateNodeByTest(ThisNode, aTest);

  FillNodeChildsByTest(ThisNode, aTest);

  UpdatePassCountResultByTest(aTest, aPassedCount, aAllTestsCount);
end;

procedure TTestConsole.TestTreeGetSelectedIndex(Sender: TObject;
  Node: TTreeNode);
begin
 Node.SelectedIndex := Node.imageindex;
end;

procedure TTestConsole.TestTreeDblClick(Sender: TObject);
var N:TTreenode;  AllTestsCount, PassedCount: integer;
  ThisTest: TQTest;
  ThisNode: TTreeNode;
begin
 N:=TestTree.Selected ;
 if assigned(N) then
  if TObject(N.data) is TQtest then
  begin
  ThisTest:=TQtest(N.data);
  Passcount.Caption := '0% Passed (0/0).';
  AllTestsCount := 1;
  PassedCount := 0;
  ThisTest.test;
  UpdateByTest(ThisTest,PassedCount,AllTestsCount);
  TestTree.FullExpand;
 end;
end;

end.

