unit WebTestSuite;

interface
uses Classes,sysutils,TestSuite;

type

TWebConsoleFormat = ( wfHTML, wfXML, wfCSV );

TWebConsole = class(TBasicConsole)                     
 private
   procedure BuildTree(s:tstringlist);
 public
   function  AsHTML:String;
   function  AsXML:String;
   function  AsCSV : String;
   procedure SaveToFile (filename:String;format:TWebConsoleFormat);
 end;

implementation

{$R *.DCR}

procedure TWebConsole.BuildTree(s:tstringlist);
var max,i,modidx,sidx:integer;thisTestSuite:TTestSuite;
    holdinfo,suites: TStringList;
begin
  holdinfo:=s;
 {add forms, object poroperty holds children, etc...}
  max:=self.SuiteCount -1;
  for i:= 0 to max do begin
      thisTestSuite:=self.Suites(i);
      modidx:=holdinfo.indexof(thisTestSuite.Owner.name);
      if modidx=-1 then modidx:=holdinfo.addobject(thisTestSuite.Owner.name,Tstringlist.create);
      suites:=TStringlist(holdinfo.objects[modidx]);
      sidx:=suites.IndexOf(thisTestSuite.name);
      if sidx=-1 then suites.addobject(thisTestSuite.name,thistestsuite);
  end;
  {Now Dump it out as H}
end;


function TWebConsole.AsHTML:String;
var tree,suites,response:tstringlist; max,jmax,kmax,i,j,k,m,mmax:integer; Suite:TTestSuite;
    Msg:String; Test:TQtest;
begin
 tree:=tstringlist.create;
 response:=tstringlist.create;
 self.execute;
 BuildTree(tree);

 max:=tree.count-1;
 for i:= 0 to max do begin
  response.add('<B>MODULE:'+tree[i]+'</B><UL>'+ #13+#10);
  suites:=TStringlist(tree.objects[i]);
  jmax:=suites.count-1;
  for j:= 0 to jmax do begin
   response.add('<LI><B>SUITE:'+suites[j]+'</B><UL>'+ #13+#10);
   Suite:=TTestSuite(suites.objects[j]);
   kmax:=Suite.Tests.Count-1;
   for k:= 0 to kmax do begin
     Test:=Suite.Tests[k];
     response.add('<LI>TEST:'+Test.TestName+':'+Test.Details+'<UL>');
     mmax:=Test.StatementCount-1;
     for m:= 0 to mmax do begin
        Case Test.statementresults(m) of
          qtUntested:MSG:='UNTESTED:';
          qtPass:MSG:='<FONT COLOR="GREEN">PASS</FONT>:';
          qtFail:MSG:='<FONT COLOR="RED">FAIL</FONT>:';
          qtException:MSG:='<FONT COLOR="PURPLE">EXCEPTION</FONT>:';
          qtLog: MSG:='<FONT COLOR="ORANGE">LOG</FONT>:';
        end;
     response.add('<LI><B>'+msg+'</B>' + Test.Statements(m) + '</LI>' + #13+#10);
     end;
     response.add('</UL></LI>');
   end;
   response.add('</UL></LI>' + #13+#10);
  end;
    response.add('</UL>' + #13+#10);
 end;
 result:=response.text;
 {cleanup}
 while tree.count>0 do begin
   TStringlist(tree.objects[0]).free;
   tree.delete(0);
   end;
 tree.free;
 response.free;
end;

//single loc for exec test.. I add timestamp info to it.

function TWebConsole.AsXML:String;
var tree,suites,response:tstringlist; max,jmax,kmax,i,j,k,m,mmax:integer; Suite:TTestSuite;
    Msg:String; Test:TQtest;
begin
 tree:=tstringlist.create;
 response:=tstringlist.create;
 self.execute;
 BuildTree(tree);

 max:=tree.count-1;
 response.add('<TESTS RUNDATE="'+Datetimetostr(Now())+'">');
 for i:= 0 to max do begin
  response.add('  <MODULE NAME="'+tree[i]+'">'+ #13+#10);
  suites:=TStringlist(tree.objects[i]);
  jmax:=suites.count-1;
  for j:= 0 to jmax do begin
   response.add('    <SUITE NAME="'+suites[j]+'">'+ #13+#10);
   Suite:=TTestSuite(suites.objects[j]);
   kmax:=Suite.Tests.Count-1;
   for k:= 0 to kmax do begin
     Test:=Suite.Tests[k];
     response.add('       <TEST NAME="'+Test.TestName +'">');
     mmax:=Test.StatementCount-1;
     for m:= 0 to mmax do begin
        Case Test.statementresults(m) of
          qtUntested:MSG:='UNTESTED';
          qtPass:MSG:='PASS';
          qtFail:MSG:='FAIL';
          qtException:MSG:='EXCEPTION';
          qtLog: MSG:='LOG';
        end;
       response.add('       <STATEMENT RESULT="'+msg+'" ID="'+inttostr(m)+'">' + Test.Statements(m) + '</STATEMENT>' + #13+#10);
     end;
       response.add('     </TEST>' + #13+#10);
   end;
       response.add('    </SUITE>' + #13+#10);
  end;
    response.add('  </MODULE>' + #13+#10);
 end;
 response.add('</TESTS>');
 result:=response.text;
 {cleanup}
 while tree.count>0 do begin
   TStringlist(tree.objects[0]).free;
   tree.delete(0);
   end;
 tree.free;
 response.free;
end;


function TWebConsole.AsCSV:String;   // from Barry Velasquez, Geoffrey Industries
var tree,suites:tstringlist; max,jmax,kmax,i,j,k,m,mmax:integer; Suite:TTestSuite;
    Msg:String; Test:TQtest; fCSVStringList:tstringlist;
begin
    tree := tstringlist.create;
    fCSVStringList:=tstringlist.create;
    self.execute;
    BuildTree(tree);

    fCSVStringList.Add('Module,Suite,Test,Results,Statements');

    max := tree.count - 1;
    for i:= 0 to max do
    begin
        suites := TStringlist(tree.objects[i]);
        jmax := suites.count - 1;
        for j:= 0 to jmax do
        begin
            Suite:=TTestSuite(suites.objects[j]);
            kmax:=Suite.Tests.Count-1;
            for k:= 0 to kmax do begin
                Test:=Suite.Tests[k];
                mmax:=Test.StatementCount-1;
                for m:= 0 to mmax do
                begin
                    Case Test.statementresults(m) of
                      qtUntested: MSG := 'UNTESTED';
                      qtPass: MSG := 'PASS';
                      qtFail: MSG := 'FAIL';
                      qtException: MSG := 'EXCEPTION';
                      qtLog: MSG := 'LOG';
                    end;

                    fCSVStringList.add( Format('%s,%s,%s,%s,"%s"',
                        [ tree[i],
                          suites[j],
                          Test.TestName+':'+Test.Details,
                          MSG,
                          Test.Statements(m) ]));
                end;
            end;
        end;
    end;

    Result := fCSVStringList.text;
    fCSVStringList.free;
    {cleanup}
    while tree.count > 0 do
    begin
        TStringlist(tree.objects[0]).free;
        tree.delete(0);
    end;
    tree.free;
end;


procedure TWebConsole.SaveToFile (filename:String;format:TWebConsoleFormat);
var outhold:tstringlist;
begin
 outhold:=tstringlist.create;
 Case format of
    wfHTML: outhold.text:=asHTML;
    wfXML:  outhold.text:=asXML;
    wfCSV:  outhold.text:=asCSV;
 end;
 outhold.savetofile(filename);
 outhold.free;
end;



end.
